//
//
// Simple, clear clock for elderly people
// Based on M5 RTC demo code
// NOTE:
//  shutdown() has no effect when charging 
//  (e.g. when connected to the USB port)!
//
// Font stored as a SPIFFS file
//
// Use Set_Clear_Clock to set the RTC, this code runs the clock
// updating the display once per minute.
//
// 
// When battery is low, display updates daily to keep
// charge to run the RTC untilk the battery is re-charged.
//
#include <M5EPD.h>
#include <SPIFFS.h>
#include <FS.h>

M5EPD_Canvas canvasD(&M5.EPD); // Canvas for Day
M5EPD_Canvas canvasT(&M5.EPD); // Canvas for time
//M5EPD_Canvas canvasB(&M5.EPD); // Canvas for battery state
M5EPD_Canvas canvas(&M5.EPD); // Placeholder for font rendering

rtc_time_t RTCtime;
rtc_date_t RTCDate;

char timeStrbuff[64];
char dayStrbuff[64];
//char batStrbuff[64];

void flushBat() {
    int32_t batVolt = M5.getBatteryVoltage(); // Only returns the right value 1st time through.....
    // but thats OK because shutdown() forces a re-start when not charging
    // (returns a low value when charging, don't know why.)
    if (batVolt < 1000) {
      sprintf(dayStrbuff, "%s", "Charging      ");
      canvasD.drawString(dayStrbuff, 0, 0);
      delay(1000);
      canvasD.pushCanvas(100, 300, UPDATE_MODE_GL16);
      delay(1000); //Wait until the display has finnished updating
    }
    else if (batVolt < 3700) {
      sprintf(timeStrbuff, "%s", "Low        ");
      sprintf(dayStrbuff, "%s", "Battery    ");
      canvasT.drawString(timeStrbuff, 0, 0);
      canvasT.pushCanvas(100, 100, UPDATE_MODE_GL16);
      canvasD.drawString(dayStrbuff, 0, 0);
      canvasD.pushCanvas(100, 300, UPDATE_MODE_GL16);
      delay(800); //Wait until the display has finnished updating
      M5.shutdown(86400); //NB Code re-starts after the shutdown time
    } 
}

void flushDate() {

    M5.RTC.getDate(&RTCDate);

    // Work out what day it is
    int dayOfWeek = zellersCongruence(RTCDate.day, RTCDate.mon, RTCDate.year);
    char arr[7][11] = {"Saturday ", "Sunday   ", "Monday   ", 
                       "Tuesday  ", "Wednesday", "Thursday  ", "Friday   "};
    
    sprintf(dayStrbuff, "%s", arr[dayOfWeek]);
    canvasD.drawString(dayStrbuff, 0, 0);
    canvasD.pushCanvas(100, 300, UPDATE_MODE_GL16);

}

void flushTime() {
   
    M5.RTC.getTime(&RTCtime);
    
    // Convert time to AM/PM
    char period[3];
    int hour12;

    if (RTCtime.hour == 0) {
        hour12 = 12;
        snprintf(period, sizeof(period), "AM ");
    } else if (RTCtime.hour == 12) {
        hour12 = 12;
        snprintf(period, sizeof(period), "PM ");
    } else if (RTCtime.hour > 12) {
        hour12 = RTCtime.hour - 12;
        snprintf(period, sizeof(period), "PM ");
    } else {
        hour12 = RTCtime.hour;
        snprintf(period, sizeof(period), "AM ");
    }

    sprintf(timeStrbuff, "%2d:%02d %s", hour12, RTCtime.min, period);
    canvasT.drawString(timeStrbuff, 0, 0);
    canvasT.pushCanvas(100, 100, UPDATE_MODE_GL16);
    
}

// What day is it?
int zellersCongruence(int day, int month, int year) {
    if (month < 3) {
        month += 12;
        year--;
    }

    int K = year % 100;
    int J = year / 100;

    int h = (day + (13 * (month + 1)) / 5 + K + K / 4 + J / 4 - 2 * J) % 7;

    // Convert h to a more conventional form 
    // (0 = Saturday, 1 = Sunday, ..., 6 = Friday)
    return h % 7;
}

void setup() {
    M5.begin(false,false,false,true,false);
    M5.EPD.SetRotation(0);
    M5.EPD.Clear(true);
    M5.RTC.begin();
    SPIFFS.begin();
    M5.BatteryADCBegin();

    canvas.loadFont("/OpenSansExtrabold.ttf", SPIFFS);  // Load font files from SPIFFS
    canvas.createRender(120);
    canvas.createRender(20);
    canvasT.createCanvas(760,200);
    canvasT.setTextSize(120);
    canvasD.createCanvas(760,200);
    canvasD.setTextSize(120);
//    canvasB.createCanvas(200,200);
//    canvasB.setTextSize(20);
}

void loop() {
    flushTime();
    flushDate();
    flushBat();
    delay(800); //Wait until the display has finnished updating
    M5.shutdown(60); //NB Code re-starts after the shutdown time
    M5.update();
    delay(100);
}